#include <amxmodx>
#include <amxmisc>
#include <fun>
#include <cstrike>
#include <fakemeta>
#include <colorchat2>

#define VIP_CONFIG "addons/amxmodx/configs/vip_system.ini"
#define MAX_PLAYERS 32
#define PREFIX_SIZE 32
#define JUMP_VELOCITY 268.328123

new bool:g_hasDoubleJump[MAX_PLAYERS + 1]
new g_extraHealth[MAX_PLAYERS + 1]
new g_szPrefix[MAX_PLAYERS + 1][32]
new bool:g_bInDoubleJump[MAX_PLAYERS + 1]

bool:is_vip_expired(const szExpiry[]) {
    new exp_year[5], exp_month[3], exp_day[3]
    new current_year[5], current_month[3], current_day[3]
    
    // Get current date
    get_time("%Y", current_year, charsmax(current_year))
    get_time("%m", current_month, charsmax(current_month))
    get_time("%d", current_day, charsmax(current_day))
    
    // Parse expiry date (format: YYYY/MM/DD)
    strtok(szExpiry, exp_year, charsmax(exp_year), exp_month, charsmax(exp_month), '/')
    strtok(exp_month, exp_month, charsmax(exp_month), exp_day, charsmax(exp_day), '/')
    
    // Compare dates
    if(str_to_num(current_year) > str_to_num(exp_year)) return true
    if(str_to_num(current_year) == str_to_num(exp_year) && str_to_num(current_month) > str_to_num(exp_month)) return true
    if(str_to_num(current_year) == str_to_num(exp_year) && str_to_num(current_month) == str_to_num(exp_month) && str_to_num(current_day) > str_to_num(exp_day)) return true
    
    return false
}


public plugin_init() {
    register_plugin("VIP System", "1.0", "Maka")
    
    register_clcmd("say", "client_say")
    register_clcmd("say_team", "client_say")
    
    register_concmd("amx_reloadvips", "cmd_reloadvips", ADMIN_RCON, "Reloads VIP configuration")
    
    // Register events
    register_event("HLTV", "event_new_round", "a", "1=0", "2=0")
    register_event("ResetHUD", "event_player_spawn", "be")
    
    // Register double jump
    register_forward(FM_PlayerPreThink, "fw_PlayerPreThink")
}

public client_putinserver(id) {
    load_vip_status(id)
}

public client_say(id) {
    if(!is_user_connected(id) || !g_szPrefix[id][0])
        return PLUGIN_CONTINUE
        
    new szName[32], szMessage[192]
    get_user_name(id, szName, charsmax(szName))
    read_args(szMessage, charsmax(szMessage))
    remove_quotes(szMessage)
    
    
    client_print_color(0, "!g[%s]!t %s!y:  %s", g_szPrefix[id], szName, szMessage)
    
    return PLUGIN_HANDLED
}

public cmd_reloadvips(id, level, cid) {
    if(!cmd_access(id, level, cid, 1))
        return PLUGIN_HANDLED
        
    reload_vip_config()
    server_print("VIP configuration reloaded!")
    
    return PLUGIN_HANDLED
}

load_vip_status(id) {
    new szAuthID[35]
    get_user_authid(id, szAuthID, charsmax(szAuthID))
    
    new szFile[64]
    get_configsdir(szFile, charsmax(szFile))
    add(szFile, charsmax(szFile), "/vip_system.ini")
    
    if(!file_exists(szFile))
        return
        
    new szData[256], szExpiry[32], szDJ[8], szHealth[8], szPrefix[32]
    new f = fopen(szFile, "rt")
    
    while(!feof(f)) {
        fgets(f, szData, charsmax(szData))
        
        if(szData[0] == ';' || szData[0] == '^n' || !szData[0])
            continue
            
        if(containi(szData, szAuthID) != -1) {
            parse(szData, szAuthID, charsmax(szAuthID), 
                          szExpiry, charsmax(szExpiry),
                          szDJ, charsmax(szDJ),
                          szHealth, charsmax(szHealth),
                          szPrefix, charsmax(szPrefix))
            
            // Check if VIP has expired
            if(is_vip_expired(szExpiry)) {
                fclose(f)
                return
            }
                          
            if(containi(szDJ, "DJ") != -1)
                g_hasDoubleJump[id] = true
                
            g_extraHealth[id] = str_to_num(szHealth)
            copy(g_szPrefix[id], PREFIX_SIZE - 1, szPrefix)
            
            break
        }
    }
    fclose(f)
}


public fw_PlayerPreThink(id) {
    if(!is_user_alive(id) || !g_hasDoubleJump[id])
        return FMRES_IGNORED
        
    new flags = pev(id, pev_flags)
    new oldbuttons = pev(id, pev_oldbuttons)
    
    // Reset double jump when player touches ground
    if(flags & FL_ONGROUND) {
        g_bInDoubleJump[id] = false
        return FMRES_IGNORED
    }
    
    // Check for jump button press
    if(!(pev(id, pev_button) & IN_JUMP) || (oldbuttons & IN_JUMP))
        return FMRES_IGNORED
    
    // Perform double jump if not already used
    if(!g_bInDoubleJump[id]) {
        g_bInDoubleJump[id] = true
        
        new Float:velocity[3]
        pev(id, pev_velocity, velocity)
        velocity[2] = JUMP_VELOCITY
        set_pev(id, pev_velocity, velocity)
    }
    
    return FMRES_IGNORED
}


public event_player_spawn(id) {
    if(!is_user_alive(id))
        return
        
    // Set extra health if player has it
    if(g_extraHealth[id] > 0) {
        set_user_health(id, g_extraHealth[id])
    }
}
public reload_vip_config() {
    // Clear existing VIP data
    for(new id = 1; id <= MAX_PLAYERS; id++) {
        g_hasDoubleJump[id] = false
        g_extraHealth[id] = 0
        g_szPrefix[id][0] = 0
    }
    
    // Reload VIP data for all connected players
    for(new id = 1; id <= MAX_PLAYERS; id++) {
        if(is_user_connected(id)) {
            load_vip_status(id)
        }
    }
    
    return PLUGIN_CONTINUE
}

public event_new_round() {
    // Reload VIP configuration each map
    reload_vip_config()
}
